import React, { ReactNode } from 'react'
import { withRouter } from 'react-router-dom'

import { RouteComponentProps } from 'react-router'
import ApiClient from '../../utils/api-client'
import withNotice, { AddNoticeProps } from '../../store/with-notice'

import RecordsTable from '../app/records-table/records-table'
import { ActionProps } from './action.props'
import RecordJSON from '../../../backend/decorators/record-json.interface'
import { ListActionResponse } from '../../../backend/actions/list-action'
import { hasForceRefresh, removeForceRefresh } from './utils/append-force-refresh'
import { Box, Pagination, Text } from '../design-system'

type State = {
  records: Array<RecordJSON>;
  page: number;
  perPage: number;
  total: number;
  loading: boolean;
  direction: 'asc' | 'desc';
  sortBy?: string;
  selectedRecords: Array<RecordJSON>;
}

type Props = ActionProps & RouteComponentProps & AddNoticeProps

// TODO: add direction enum

/**
 * @name NewAction
 * @category Actions
 * @description Shows form for creating a given record.
 * @component
 * @private
 */
class List extends React.Component<Props, State> {
  private blockRefresh: boolean;

  constructor(props) {
    super(props)
    this.handleActionPerformed = this.handleActionPerformed.bind(this)
    this.handleSelect = this.handleSelect.bind(this)
    this.handleSelectAll = this.handleSelectAll.bind(this)
    this.onPaginationChange = this.onPaginationChange.bind(this)
    this.blockRefresh = false
    this.state = {
      records: [],
      page: 1,
      perPage: 20,
      total: 0,
      loading: false,
      direction: 'asc',
      sortBy: undefined,
      selectedRecords: [],
    }
  }

  componentDidMount(): void {
    this._fetchData(this.props)
  }

  shouldComponentUpdate(newProps): boolean {
    const { resource, location, history } = this.props
    if (hasForceRefresh(newProps.location.search)) {
      this._fetchData(newProps)
      this.blockRefresh = true
      history.replace([
        newProps.location.pathname,
        removeForceRefresh(newProps.location.search).toString(),
      ].join('?'))
      return false
    }

    if (resource.id !== newProps.resource.id
       || location.search !== newProps.location.search) {
      this._fetchData(newProps)
      return false
    }
    return true
  }

  componentWillUnmount(): void {
    const { setTag } = this.props
    if (setTag) {
      setTag('')
    }
  }

  onPaginationChange(pageNumber): void {
    const { location, history } = this.props
    const search = new URLSearchParams(location.search)
    search.set('page', pageNumber.toString())
    history.push({ search: search.toString() })
  }

  _fetchData(props: Props): void {
    const { location, resource, setTag, addNotice } = props
    const { resource: oldResource } = this.props
    const { selectedRecords } = this.state
    if (this.blockRefresh) {
      this.blockRefresh = false
      return
    }

    const api = new ApiClient()
    const shouldPreserveSelected = oldResource.id === resource.id
      && !hasForceRefresh(props.location.search)
    this.setState({ loading: true })
    const query = new URLSearchParams(location.search)
    api.resourceAction({
      actionName: 'list',
      resourceId: resource.id,
      params: query,
    }).then((response) => {
      const listActionResponse = response.data as ListActionResponse
      this.setState({
        records: listActionResponse.records,
        page: listActionResponse.meta.page,
        perPage: listActionResponse.meta.perPage,
        total: listActionResponse.meta.total,
        direction: listActionResponse.meta.direction,
        sortBy: listActionResponse.meta.sortBy,
        selectedRecords: shouldPreserveSelected ? selectedRecords : [],
        loading: false,
      })
      if (setTag) {
        if (typeof response.data.meta.total === 'undefined') {
          setTag('')
        } else {
          setTag(response.data.meta.total.toString())
        }
      }
    }).catch(() => {
      addNotice({
        message: 'There was an error fetching records, Check out console to see more information.',
        type: 'error',
      })
    })
  }

  handleActionPerformed(): void {
    this._fetchData(this.props)
  }

  handleSelect(record: RecordJSON): void {
    const { selectedRecords } = this.state
    const selectedIndex = selectedRecords.findIndex(selected => selected.id === record.id)
    if (selectedIndex < 0) {
      this.setState({ selectedRecords: [...selectedRecords, record] })
    } else {
      const newSelectedRecords = [...selectedRecords]
      newSelectedRecords.splice(selectedIndex, 1)
      this.setState({ selectedRecords: newSelectedRecords })
    }
  }

  handleSelectAll(): void {
    const { records, selectedRecords } = this.state

    const missing = records.filter(record => (
      !selectedRecords.find(selected => selected.id === record.id)
      && record.bulkActions.length
    ))
    if (missing.length) {
      this.setState({ selectedRecords: [...selectedRecords, ...missing] })
    } else {
      const newSelectedRecords = selectedRecords.filter(selected => (
        !records.find(record => record.id === selected.id)
      ))
      this.setState({ selectedRecords: newSelectedRecords })
    }
  }

  render(): ReactNode {
    const { resource } = this.props
    const {
      records, page, perPage, total,
      loading, direction, sortBy, selectedRecords,
    } = this.state
    return (
      <Box variant="white">
        <RecordsTable
          resource={resource}
          records={records}
          actionPerformed={this.handleActionPerformed}
          onSelect={this.handleSelect}
          onSelectAll={this.handleSelectAll}
          selectedRecords={selectedRecords}
          direction={direction}
          sortBy={sortBy}
          isLoading={loading}
        />
        <Text mt="xl" textAlign="center">
          <Pagination
            page={page}
            perPage={perPage}
            total={total}
            onChange={this.onPaginationChange}
          />
        </Text>
      </Box>
    )
  }
}

export default withNotice(withRouter(List))
